﻿/*
Copyright (c) 2010 Lance Ward
This software is licensed under the MIT license, except where otherwise noted.
For complete license details, refer to license.txt
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using ADNLib;

namespace ADNCanvas
{
	public class ToolFreehand : Tool
	{
		//handles type spray and freehand
		public ToolFreehand(DrawingToolType toolType)
		{
			this.toolType = toolType;
		}

		public ToolFreehand() { }

		public override void MouseDown(PointF location, System.Windows.Forms.MouseButtons button, int clicks)
		{
			if (currentShape == null)
			{
				CreateShape(location);
			}
		}

		public override void MouseMove(PointF location, System.Windows.Forms.MouseButtons button, int clicks)
		{
			if (currentShape != null)
			{
				currentShape.AppendPoint(location, false);
			}
		}

		public override void MouseUp(PointF location, System.Windows.Forms.MouseButtons button, int clicks)
		{
			currentShape.AppendPoint(location, false);
			OnShapeComplete(true);
			OnToolComplete(true);

			currentShape = null;
		}

		public override System.Windows.Forms.Cursor GetToolCursor()
		{
			return Cursors.Cross;
		}

		private void CreateShape(PointF startPoint)
		{
			switch (ToolType)
			{
				case DrawingToolType.Freehand:
					currentShape = new ShapeFreeHand();
					break;
				case DrawingToolType.Spray:
					currentShape = new ShapeSpray();
					break;
			}

			currentShape.AppendPoint(startPoint, true);
			currentShape.FillColor = defaultFill.Clone();
			currentShape.LineColor = defaultLine.Clone();
		}

		protected override void OnShapeComplete(bool isComplete)
		{
			base.OnShapeComplete(isComplete);
		}

		protected override void OnToolComplete(bool isComplete)
		{
			base.OnToolComplete(isComplete);
		}

	}
}
